@@ -6,6 +6,7 @@ import time
 from datetime import datetime, timedelta, timezone
 from typing import List
 from io import StringIO
+import sys
 
 import ffmpeg
 import tqdm
@@ -21,6 +22,7 @@ class Recording():
         self.cname = cname
         self.url = url
         self.startTime = startTime
+
     def __str__(self) -> str:
         return "{}-{}".format(self.cname, self.startTime)
 
@@ -46,8 +48,8 @@ def parse_args():
                         help='create a separate folder per camera/duration (default: disabled)')
     parser.add_argument('--debug', action=argparse.BooleanOptionalAction, dest="debug",
                         help='enable debug mode (default: false)')
-    parser.add_argument('--videoformat', dest="videoformat", default="mkv", choices=['mkv', 'mp4', 'avi'],
-                        help='specify video format (default: mkv)')
+    parser.add_argument('--videoformat', dest="videoformat", default="mp4", choices=['mkv', 'mp4', 'avi'],
+                        help='specify video format (default: mp4)')
     parser.add_argument('--downloads', dest="downloads", default=os.path.join(os.getcwd(), "Downloads"),
                         help='the downloads folder (default: "Downloads")')
     parser.add_argument('--frames', dest="frames", type=int,
@@ -78,7 +80,9 @@ def parse_args():
                         help='enable experimental downloading of saved photos')
     parser.add_argument('--mock', dest="mock", action=argparse.BooleanOptionalAction,
                         help='enable mock mode  WARNING! This will not download anything from the server')
-    parser.add_argument('--ui', dest="ui", action=argparse.BooleanOptionalAction, default=False,
+    parser.add_argument('--ui', dest="ui", action=argparse.BooleanOptionalAction,
+                        # If running under PyInstaller, use the UI
+                        default=bool(getattr(sys, 'frozen', False)),
                         help='enable UI interface WARNING! Requires Qt5 to be installed')
     args = parser.parse_args()
     return args
@@ -126,38 +130,73 @@ def video_download_from_channel(server: hikvisionapi.HikvisionServer, args, url,
                 protocol=False, credentials=True))
             hikvisionapi.downloadRTSPOnlyFrames(
                 url, name, debug=args.debug, force=args.force, modulo=args.frames, skipSeconds=args.skipseconds, seconds=args.seconds)
-        except ffmpeg.Error:
+        except ffmpeg.Error as e:
             logging.error(
                 "Could not download %s. Try to remove --frames." % name)
+            logging.error(e)
     if args.ffmpeg:
         try:
             url = url.replace(server.host, server.address(
                 protocol=False, credentials=True))
             hikvisionapi.downloadRTSP(
                 url, name, debug=args.debug, force=args.force, skipSeconds=args.skipseconds, seconds=args.seconds)
-        except ffmpeg.Error:
+        except ffmpeg.Error as e:
             logging.error(
                 "Could not download %s. Try to remove --fmpeg." % name)
+            logging.error(e)
     else:
         if args.folders:
             temporaryname = "%s.mp4" % filename
         else:
             temporaryname = "%s-%s.mp4" % (filename, cid)
-        r = server.ContentMgmt.search.downloadURI(url)
+        try:
+            r = server.ContentMgmt.search.downloadURI(url)
+        except hikvisionapi.HikvisionError as e:
+            try:
+                supports = server.ContentMgmt.search.get_download_capabilities()
+                logging.debug(f'Device capabilities: {supports}')
+                if supports['DownloadAbility']['isSupportDownloadbyFileName'] == 'false':
+                    logging.error(
+                        "Downloading by file name is not supported for this device.")
+                    logging.error(
+                        "Try to add --ffmpeg to force recording the videos.")
+                    logging.error(e)
+                    return
+                if supports['DownloadAbility']['isSupportDownloadbyTime'] == 'false':
+                    logging.error(
+                        "Downloading by time is not supported for this device.")
+                    logging.error(
+                        "Try to add --ffmpeg to force recording the videos.")
+                    logging.error(e)
+                    return
+            except (hikvisionapi.HikvisionError, TypeError) as e:
+                logging.error(
+                    "Could not get download capabilities. The device dosen't seem to support getting capabilities.")
+                logging.error(
+                    "Try to add --ffmpeg to force recording the videos.")
+                logging.error(e)
+                return
+            logging.error(
+                "Could not download %s. Try to add --ffmpeg." % name)
+            logging.error(e)
+            return
         open(temporaryname, 'wb').write(r.content)
         try:
+            logging.info(args)
             hikvisionapi.processSavedVideo(
                 temporaryname, debug=args.debug, skipSeconds=args.skipseconds, seconds=args.seconds,
                 fileFormat=args.videoformat, forceTranscode=args.forcetranscoding)
-        except ffmpeg.Error:
+        except ffmpeg.Error as e:
             logging.error(
                 "Could not transcode %s. Try to remove --forcetranscoding." % name)
+            logging.error(e)
     if os.environ.get('RUNNING_IN_DOCKER') == 'TRUE':
         os.chmod(name, 0o777)
     end_time = time.perf_counter()
     run_time = end_time - start_time
     logging.info(f"Finished downloading {name} in {run_time:.2f} seconds")
 
+
 def search_for_recordings(server: hikvisionapi.HikvisionServer, args) -> List[Recording]:
     start_time = time.perf_counter()
     channelids = []
@@ -170,15 +209,20 @@ def search_for_recordings(server: hikvisionapi.HikvisionServer, args) -> List[Re
                 'channelName': str(cid),
             })
     else:
-        channelList = server.Streaming.getChannels()
-        for channel in channelList['StreamingChannelList']['StreamingChannel']:
-            if (int(channel['id']) % 10 == 1):
-                if args.photos:
-                    # Force looking at the hidden 103 channel for the photos
-                    channel['id'] = str(int(channel['id'])+2)
-                channelids.append(channel['id'])
-                channels.append(channel)
-            logging.info("Found channels %s" % channelids)
+        try:
+            channelList = server.Streaming.getChannels()
+            for channel in channelList['StreamingChannelList']['StreamingChannel']:
+                if (int(channel['id']) % 10 == 1):
+                    if args.photos:
+                        # Force looking at the hidden 103 channel for the photos
+                        channel['id'] = str(int(channel['id'])+2)
+                    channelids.append(channel['id'])
+                    channels.append(channel)
+                logging.info("Found channels %s" % channelids)
+        except HikvisionException as e:
+            logging.error(
+                "Could not get channel list. If you still want to continue, add the argument --cameras with the channel ids you want to download.")
+            raise e
 
     downloadQueue = []
     for channel in channels:
@@ -210,8 +254,9 @@ def search_for_recordings(server: hikvisionapi.HikvisionServer, args) -> List[Re
                     cid, starttime.isoformat() + "Z", endtime.isoformat() + "Z")
                 logging.info("Found %s recordings for channel %s" %
                              (recordings['CMSearchResult']['numOfMatches'], cid))
-        except hikvisionapi.classes.HikvisionException:
+        except hikvisionapi.classes.HikvisionException as e:
             logging.error("Could not get recordings for channel %s" % cid)
+            logging.error(e)
             continue
 
         # This loops from every recording
@@ -225,11 +270,11 @@ def search_for_recordings(server: hikvisionapi.HikvisionServer, args) -> List[Re
         result = []
         for i in recordinglist:
             result.append(Recording(
-                cid=cid, 
-                cname=cname, 
+                cid=cid,
+                cname=cname,
                 url=i['mediaSegmentDescriptor']['playbackURI'],
                 startTime=i['timeSpan']['startTime']
-                ))
+            ))
             logging.debug("Found recording type %s on channel %s" % (
                 i['mediaSegmentDescriptor']['contentType'], cid
             ))
@@ -239,19 +284,30 @@ def search_for_recordings(server: hikvisionapi.HikvisionServer, args) -> List[Re
         downloadQueue.extend(result)
     end_time = time.perf_counter()
     run_time = end_time - start_time
-    logging.info(f"Found {len(downloadQueue)} files to download in {run_time:.2f} seconds")
+    logging.info(
+        f"Found {len(downloadQueue)} files to download in {run_time:.2f} seconds")
     return downloadQueue
 
-def search_for_recordings_mock() -> List[Recording]:
+
+def search_for_recordings_mock(args) -> List[Recording]:
+    logger = logging.getLogger('hikload')
+    logger.debug(f"{args=}")
     return [
-        Recording(cid=1, cname="Channel 1", startTime="2021-12-19T09:04:46Z", url="https://tedyst.ro"),
-        Recording(cid=1, cname="Channel 1", startTime="2021-12-19T09:04:47Z", url="https://tedyst.ro"),
-        Recording(cid=1, cname="Channel 1", startTime="2021-12-19T09:04:48Z", url="https://tedyst.ro"),
-        Recording(cid=1, cname="Channel 1", startTime="2021-12-19T09:04:49Z", url="https://tedyst.ro"),
-        Recording(cid=1, cname="Channel 1", startTime="2021-12-19T09:04:50Z", url="https://tedyst.ro"),
-        Recording(cid=1, cname="Channel 1", startTime="2021-12-19T09:04:51Z", url="https://tedyst.ro"),
+        Recording(cid=1, cname="Channel 1",
+                  startTime="2021-12-19T09:04:46Z", url="https://tedyst.ro"),
+        Recording(cid=1, cname="Channel 1",
+                  startTime="2021-12-19T09:04:47Z", url="https://tedyst.ro"),
+        Recording(cid=1, cname="Channel 1",
+                  startTime="2021-12-19T09:04:48Z", url="https://tedyst.ro"),
+        Recording(cid=1, cname="Channel 1",
+                  startTime="2021-12-19T09:04:49Z", url="https://tedyst.ro"),
+        Recording(cid=1, cname="Channel 1",
+                  startTime="2021-12-19T09:04:50Z", url="https://tedyst.ro"),
+        Recording(cid=1, cname="Channel 1",
+                  startTime="2021-12-19T09:04:51Z", url="https://tedyst.ro"),
     ]
 
+
 def download_recording(server: hikvisionapi.HikvisionServer, args, recordingobj: Recording, original_path):
     try:
         logger = logging.getLogger('hikload')
@@ -308,13 +364,17 @@ def download_recordings(server: hikvisionapi.HikvisionServer, args, downloadQueu
     for recordingobj in tqdm.tqdm(downloadQueue):
         download_recording(server, args, recordingobj, original_path)
 
+
 def run(args):
     if args.server == "" or args.server == None:
-        raise HikvisionException("No server specified! You need to specify a server with --server")
+        raise HikvisionException(
+            "No server specified! You need to specify a server with --server")
     if args.username == "" or args.username == None:
-        raise HikvisionException("No username specified! You need to specify a username with --username")
+        raise HikvisionException(
+            "No username specified! You need to specify a username with --username")
     if args.password == "" or args.password == None:
-        raise HikvisionException("No password specified! You need to specify a password with --password")
+        raise HikvisionException(
+            "No password specified! You need to specify a password with --password")
 
     server = hikvisionapi.HikvisionServer(
         args.server, args.username, args.password)
@@ -330,7 +390,7 @@ def run(args):
     with logging_redirect_tqdm():
         server.test_connection()
         if args.mock:
-            downloadQueue = search_for_recordings_mock()
+            downloadQueue = search_for_recordings_mock(args)
             args.skipdownload = True
         else:
             downloadQueue = search_for_recordings(server, args)
